from q19_nlp.prompts import PromptManager
from q19_nlp.core import LLMProvider


class LLMGenerator(object):
    """
    A class to generate responses to users using a Language Learning Model (LLM) provider.

    Parameters
    ----------
    llm_provider : LLMProvider
        The LLM provider instance used for generating responses.

    Attributes
    ----------
    llm : LLMProvider
        The LLM provider instance.
    pm : PromptManager
        The prompt manager instance for formatting prompts.
    """

    def __init__(self, llm_provider: LLMProvider):
        self.llm = llm_provider
        self.pm = PromptManager()

    def generate(
            self,
            field: str,
            lang: str,
            description: str | None,
            history: list[dict[str, str]] | None
    ) -> tuple[str, list[dict[str, str]]]:
        """
        Generate a response using the LLM based on given parameters.

        Parameters
        ----------
        field : str
            The field for which to generate a response.
        lang : str
            The language in which to generate the response.
        description : str or None
            Additional description or context for the generation, like type or smth like taht
        history : list[dict[str, str]] or None
            Previous conversation history as a list of dictionaries. Same as OpenAI API format

        Returns
        -------
        tuple[str, list[dict[str, str]]]
            A tuple containing:
            - The generated response as a string
            - The updated conversation history as a list of dictionaries
        """
        history = self.pm.field_asking_prompt(
            field=field,
            lang=lang,
            description=description,
            history=history
        )  # formats history
        response, new_history = self.llm.get_completion(history)
        return response, new_history
    
    def get_init_history(self) -> list[dict[str, str]]:
        """
        Get initial history for the LLM.
        """
        return self.pm.get_init_generation_history()
