from typing import Any

from q19_nlp.prompts import PromptManager
from q19_nlp.postprocessor import BaseJsonPostprocessor
from q19_nlp.core import LLMProvider


class LLMParser(object):
    """
    A parser that uses LLM to extract structured information from text.

    Parameters
    ----------
    llm_provider : LLMProvider
        Provider for LLM completion services.

    Attributes
    ----------
    llm : LLMProvider
        Instance of LLM provider for completions.
    pm : PromptManager
        Manager for handling prompt templates.
    postprocessor : BaseJsonPostprocessor
        Processor for parsing JSON responses.
    """

    def __init__(self, llm_provider: LLMProvider):
        self.llm = llm_provider
        self.pm = PromptManager()
        self.postprocessor = BaseJsonPostprocessor()

    def extract(
            self,
            history: list[dict[str, str]],
            field_names: dict[str, str],
            message: str
    ) -> dict[str, Any]:
        """
        Extract structured information from a message using LLM.

        Parameters
        ----------
        history : list[dict[str, str]]
            Chat history containing previous interactions. Same as OpenAI API
        field_names : dict[str, str]
            Dictionary mapping field names to their types. E.g. schema
        message : str
            Input message to extract information from.

        Returns
        -------
        tuple[dict[str, Any], list[dict[str, str]]]
            A tuple containing:
            - Dictionary of extracted fields and their values
            - Updated chat history after extraction
        """
        history = self.pm.field_extraction_prompt(
            input_schema=field_names,
            user_input=message,
            history=history
        )
        
        response, new_history = self.llm.get_completion(history)
        extracted = self.postprocessor.process(response, field_names)
        
        fields = {}
        for field in field_names:
            extract = extracted.get(field, "")
            if extract != "":
                fields[field] = extract
        return fields, new_history
    
    def get_init_history(self) -> list[dict[str, str]]:
        """
        Get initial history for the LLM.
        """
        return self.pm.get_init_extraction_history()
