import json

from typing import Any


class BaseJsonPostprocessor(object):
    def process(self, response: str, field_names: dict[str, str]) -> dict[str, Any]:
        """
        Process a string response containing JSON-like data and extract specified fields.

        Parameters
        ----------
        response : str
            The input string containing JSON or JSON-like data, potentially wrapped in
            markdown code blocks (LLMs do that sometimes).
        field_names : dict[str, str]
            A dictionary mapping field names to process.

        Returns
        -------
        dict[str, Any]
            A dictionary containing the extracted field values. If JSON parsing fails,
            only the fields specified in field_names will be extracted.

        Notes
        -----
        The method first attempts to parse the input as valid JSON. If that fails,
        it falls back to line-by-line parsing looking for key-value pairs separated
        by colons.
        """
        cleaned_response = response.replace("```json", "").replace("```", "").strip()
        
        # Try JSON parsing first
        try:
            extracted = json.loads(cleaned_response)
            
        # Fallback to manual parsing if JSON fails
        except json.JSONDecodeError:
            extracted = {}
            for line in cleaned_response.split('\n'):
                if ':' in line:
                    key, value = line.split(':', 1)
                    key = key.strip().strip('"').strip("'")
                    value = value.strip().strip('"').strip("'").rstrip(',')
                    if key in field_names:
                        extracted[key] = value

        return extracted