from typing import Any
import yaml
import os

def format_message(message: str, role: str) -> dict[str, str]:
    """Format a message with its role into a dictionary.

    Parameters
    ----------
    message : str
        The content of the message
    role : str
        The role of the message sender (e.g., 'user', 'system', 'assistant')

    Returns
    -------
    dict[str, str]
        A dictionary containing the role and content
    """
    return {"role": role, "content": message}


def extractor(
        user_input: str, 
        input_schema: dict[str, str], 
        role: str
) -> dict[str, str]:
    """Create an extraction prompt from user input and schema.

    Parameters
    ----------
    user_input : str
        The input text from the user
    input_schema : dict[str, str]
        Schema defining the expected input fields
    role : str
        The role of the message sender

    Returns
    -------
    dict[str, str]
        Formatted message dictionary with the extraction prompt
    """
    # TODO: change prompts
    # TODO: jinja2 template
    prompt_template = f"Potential Fields: {input_schema}\nMessage: {user_input}"
    return format_message(prompt_template, role)


def generator(
        field: str, 
        lang: str, 
        description: str | None, 
        role: str
) -> dict[str, str]:
    """Create a generation prompt for a specific field.

    Parameters
    ----------
    field : str
        The field to generate content for
    lang : str
        The target language
    description : str or None
        Optional description of the field
    role : str
        The role of the message sender

    Returns
    -------
    dict[str, str]
        Formatted message dictionary with the generation prompt
    """
    # TODO: change prompts
    # TODO: jinja2 template
    prompt_template = f"Field: {field}\nLanguage: {lang}\nDescription: {description}"
    return format_message(prompt_template, role)


class PromptManager(object):
    """Manages prompt generation and field extraction for NLP tasks.

    Attributes
    ----------
    system_extract_prompt : Any
        System prompt for extraction loaded from YAML
    system_generate_prompt : Any
        System prompt for generation loaded from YAML
    """

    def __init__(self):
        """Initialize the PromptManager with system prompts."""
        root_file = os.path.dirname(os.path.abspath(__file__))
        self.system_extract_prompt = yaml.safe_load(open(f"{root_file}/sysp/extractor.yaml"))
        self.system_generate_prompt = yaml.safe_load(open(f"{root_file}/sysp/generator.yaml"))

    def field_extraction_prompt(
            self,
            input_schema: dict[str, str],
            user_input: str,
            history: list[dict[str, str]] | None,
    ) -> dict[str, Any]:
        """Create a prompt for field extraction from user input.

        Parameters
        ----------
        input_schema : dict[str, str]
            Schema defining the expected input fields
        user_input : str
            The input text from the user
        history : list[dict[str, str]] or None
            Optional conversation history if none creates one

        Returns
        -------
        dict[str, Any]
            Updated conversation history with the extraction prompt
        """
        if history is None:
            history = []
            history.append(format_message(self.system_extract_prompt, "system"))
        history.append(extractor(user_input, input_schema, "user"))  # TODO: implement
        return history

    def field_asking_prompt(
            self,
            field: str,
            lang: str,
            description: str | None,
            history: list[dict[str, str]] | None
    ) -> str:
        """Create a prompt for asking about a specific field.

        Parameters
        ----------
        field : str
            The field to ask about
        lang : str
            The target language
        description : str or None
            Optional description of the field
        history : list[dict[str, str]] or None
            Optional conversation history if none creates one

        Returns
        -------
        str
            Updated conversation history with the field-specific prompt
        """
        if history is None:
            history = []
            history.append(format_message(self.system_generate_prompt, "system"))
        history.append(generator(field, lang, description, "user"))  # TODO: implement
        return history
    
    def get_init_generation_history(self) -> list[dict[str, str]]:
        """
        Get initial history for the LLM.
        """
        return [format_message(self.system_generate_prompt, "system"),]
    
    def get_init_extraction_history(self) -> list[dict[str, str]]:
        """
        Get initial history for the LLM.
        """
        return [format_message(self.system_extract_prompt, "system"),]
